library bloc_package;

import 'package:flutter/material.dart';
import 'package:rxdart/rxdart.dart';

Type _typeOf<T>() => T;

abstract class BaseBloc {
  void dispose();
}

class BaseLoadingBloc extends BaseBloc {
  final _isLoading = BehaviorSubject<bool>.seeded(false);
  Stream<bool> get isLoadingStream => _isLoading.stream;

  void setLoading(bool isLoading) {
    _isLoading.sink.add(isLoading);
  }

  @override
  void dispose() {
    _isLoading.close();
  }
}

class BlocProvider<T extends BaseBloc> extends StatefulWidget {
  BlocProvider({
    Key key,
    @required this.child,
    @required this.bloc,
  }) : super(key: key);

  final Widget child;
  final T bloc;

  @override
  _BlocProviderState<T> createState() => _BlocProviderState<T>();

  static T of<T extends BaseBloc>(BuildContext context) {
    final type = _typeOf<_BlocProviderInherited<T>>();
    _BlocProviderInherited<T> provider =
        context.ancestorInheritedElementForWidgetOfExactType(type)?.widget;
    return provider?.bloc;
  }
}

class _BlocProviderState<T extends BaseBloc> extends State<BlocProvider<T>> {
  @override
  void dispose() {
    widget.bloc?.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return new _BlocProviderInherited<T>(
      bloc: widget.bloc,
      child: widget.child,
    );
  }
}

class _BlocProviderInherited<T> extends InheritedWidget {
  _BlocProviderInherited({
    Key key,
    @required Widget child,
    @required this.bloc,
  }) : super(key: key, child: child);

  final T bloc;

  @override
  bool updateShouldNotify(_BlocProviderInherited oldWidget) => false;
}